#' GetDataSummaryArtifacts
#'
#' Produces plots and artifacts for the data summary page
#'
#' @param inputData Input data as data.table. Required
#'
#' @return list of objects
#'
#' @examples
#' \dontrun{
#' GetDataSummaryArtifacts(inputData)
#' }
#'
#' @export
GetDataSummaryArtifacts <- function(inputData)
{
  inputData <- copy(inputData)

  missPlot <- PlotMissingnessSummary(inputData)

  # New stuff
  inputData[, ":="(
    NotificationTime = DateOfNotificationYear + 1/4 * DateOfNotificationQuarter,
    DiagnosisTime = DateOfDiagnosisYear + 1/4 * DateOfDiagnosisQuarter
  )]
  inputData[, VarX := 4 * (NotificationTime - DiagnosisTime)]
  inputData[VarX < 0, VarX := NA]

  # Info to display
  missingCount <- inputData[, c(
    NotificationTime = sum(is.na(NotificationTime)),
    DiagnosisTime = sum(is.na(DiagnosisTime)),
    VarX = sum(is.na(VarX))
  )]

  # Observed delay distribution
  if (!all(is.na(inputData$VarX))) {
    quant95 <- quantile(inputData$VarX, probs = 0.95, na.rm = TRUE)
    delayDensFullPlot <- ggplot() +
      geom_density(data = inputData, mapping = aes(VarX), adjust = 4) +
      geom_vline(xintercept = quant95, color = "red") +
      annotate("text",
               label = paste("95% of cases reported \nby", quant95, "quarters"),
               x = quant95 + 0.5,
               y = -Inf,
               hjust = 0,
               vjust = -1,
               color = "red") +
      coord_cartesian(xlim = c(0, 30)) +
      theme(text = element_text(size = 14)) +
      ggtitle("Density of Reporting Delay\n") +
      xlab("Delay in quarters of the year") +
      ylab("Proportion reported with the delay")

    shortData <- inputData[DateOfDiagnosisYear < max(DateOfDiagnosisYear) - 5]
    quant95 <- quantile(shortData$VarX, probs = 0.95, na.rm = TRUE)
    delayDensShortPlot <- ggplot() +
      geom_density(data = shortData, mapping = aes(VarX), adjust = 4) +
      geom_vline(xintercept = quant95, color = "red") +
      annotate("text",
               label = paste("95% of cases reported \nby", quant95, "quarters"),
               x = quant95 + 0.5,
               y = -Inf,
               hjust = 0,
               vjust = -1,
               color = "red") +
      coord_cartesian(xlim = c(0, 30)) +
      theme(text = element_text(size = 14)) +
      ggtitle("Density of Reporting Delay\nLast 5 years deleted") +
      xlab("Delay in quarters of the year") +
      ylab("Proportion reported with the delay")
  } else {
    delayDensFullPlot <- NULL
    delayDensShortPlot <- NULL
  }

  # Mean delay plot
  meanDelay <- inputData[!is.na(NotificationTime),
                         .(MeanDelay = mean(VarX, na.rm = TRUE)),
                         by = .(NotificationTime)]

  if (nrow(meanDelay) > 0) {
    meanDelay[, rsd := RollingApply(MeanDelay, 10, sd)]
    meanDelayCurve <- meanDelay[, as.data.table(lowess(NotificationTime, MeanDelay))]
    meanDelayUpper <- meanDelay[, as.data.table(lowess(NotificationTime, MeanDelay + 1.96 * rsd))]
    meanDelayPlot <- ggplot() +
      geom_point(data = meanDelay, aes(NotificationTime, MeanDelay)) +
      geom_line(data = meanDelayCurve, aes(x, y, color = "Smoothed mean reporting delay"), size = 0.5) +
      geom_line(data = meanDelayUpper, aes(x, y, color = "Expected upper bound"), linetype = "dashed", size = 0.5) +
      labs(colour = "Datasets", x = "xxx", y = "yyy") +
      scale_color_manual(values = c("blue", "red")) +
      theme(text = element_text(size = 14)) +
      ggtitle("Reporting delay by notification quarter") +
      xlab("Notification time in quarters of the year") +
      ylab("Mean reporting delay")
  } else {
    meanDelayPlot <- NULL
  }

  return(list(MissPlot = missPlot,
              DelayDensFullPlot = delayDensFullPlot,
              DelayDensShortPlot = delayDensShortPlot,
              MeanDelayPlot = meanDelayPlot))
}
